// Copyright (c) Microsoft. All rights reserved.
// Licensed under the MIT license. See License.txt in the project root.

package com.microsoft.alm.authentication;

import com.microsoft.alm.secret.Token;

import java.net.URI;
import java.util.concurrent.atomic.AtomicReference;

public interface ITokenStore
{
    /**
     * Deletes a {@link Token} from the underlying storage.
     *
     * @param targetUri The key identifying which token is being deleted.
     */
    void deleteToken(final URI targetUri);

    /**
     * Reads a {@link Token} from the underlying storage.
     *
     * @param targetUri The key identifying which token to read.
     * @param token A {@link Token} if successful; otherwise null.
     * @return True if successful; otherwise false.
     */
    boolean readToken(final URI targetUri, final AtomicReference<Token> token);

    /**
     * Writes a {@link Token} to the underlying storage.
     *
     * @param targetUri Unique identifier for the token, used when reading back from storage.
     * @param token The {@link Token} to be written.
     */
    void writeToken(final URI targetUri, final Token token);
}
